#!/usr/bin/python
# -*- coding: utf-8 -*-
# nautilus-pastebin - Nautilus extension to paste a file to a pastebin service
#
# Copyright (C) 2009-2012 Alessio Treglia <quadrispro@ubuntu.com>
#
# This package is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This package is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this package; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
#

import os
import sys
import time
import gettext
import re
import urllib
import webbrowser
from subprocess import check_output, CalledProcessError
from threading import Thread

from gi.repository import GObject, Gdk, Gio, Gtk
from gi.repository import Nautilus

try:
    from gi.repository import Notify
except ImportError:
    Notify = None

# Globals
APP_NAME = "nautilus-pastebin"
LOGGING = 1
MAINTAINER_MODE = True

gettext.install(APP_NAME)

def get_presets():
    try:
        res = check_output(["pastebinit", "-l"])
        res = re.split("\n-?\s?", res.strip())[1:]
    except:
        return None
    return res

class log:
    def log(self, message):
        if self.logger:
            self.logger.debug(message)
    def __init__(self):
        logger = None
        if LOGGING:
            import logging
            logger = logging.getLogger(APP_NAME)
            logger.setLevel(logging.DEBUG)
            ch = logging.StreamHandler()
            ch.setLevel(logging.DEBUG)
            logger.addHandler(ch)
        self.logger = logger

log = log().log

Notify.init("Nautilus pastebin extension")

class PastebinThread(Thread):
    def __init__(self, settings, filename):
        self.settings = settings
        self.filename = filename
        Thread.__init__(self)

    def run(self):
        log ("PastebinThread started!")
        cmdline = ["pastebinit"]
        
        options = {}
        options['-b'] = "http://" + self.settings.get_string("pastebin")
        options['-a'] = self.settings.get_string("author")
        options['-u'] = self.settings.get_string("username")
        options['-p'] = self.settings.get_string("password")
        
        if not options['-a'] or options['-a'].strip() == '':
            options['-a'] = os.getenv("USERNAME")
        
        for opt in options:
            if options[opt] != None and options[opt].strip() != '':
                cmdline.append(opt)
                cmdline.append(options[opt])

        cmdline.append(self.filename)

        summary = ''
        message = ''
        icon = None
        helper = Gtk.Button()

        try:
            pasteurl = check_output(cmdline).strip()
        except CalledProcessError as error:
            # error.cmd and error.returncode, pasteurl has strerr
            summary = error.message
            message = pasteurl
            icon = helper.render_icon(Gtk.STOCK_DIALOG_ERROR, Gtk.IconSize.DIALOG, None)
        except:
#        if not pasteurl or not re.match("^http://.*$", pasteurl):
            summary = _("Unable to read or parse the result page.")
            message = _("It could be a server timeout or a change server side. Try later.")
            icon = helper.render_icon(Gtk.STOCK_DIALOG_ERROR, Gtk.IconSize.DIALOG, None)
        else:
            summary = 'File pasted to: '
            #URLmarkup = '<a href="%(pasteurl)s">%(pasteurl)s</a>'
            #message = URLmarkup % {'pasteurl':pasteurl}
            message = pasteurl
            icon = helper.render_icon(Gtk.STOCK_PASTE, Gtk.IconSize.DIALOG, None)

            atom = Gdk.atom_intern('CLIPBOARD', True)
            cb = Gtk.Clipboard.get(atom)
            cb.clear()
            cb.set_text(pasteurl, -1)
            
            # Open a browser window
            if self.settings['openbrowser']:
                webbrowser.open(pasteurl)
            
        # Show a bubble
        if self.settings['shownotification']:
            if Notify != None:
                n = Notify.Notification.new(summary, message, None)
                n.set_image_from_pixbuf(icon)
                n.show()
            else:
                log ("libnotify is not installed")

class PastebinitExtension(GObject.GObject, Nautilus.MenuProvider):
    BASE_KEY = "apps.nautilus-pastebin"
    def __init__(self):
        log ("Intializing nautilus-pastebin extension...")
        self.settings = Gio.Settings.new(self.BASE_KEY)
        self.presets = get_presets()
        if not self.presets:
            log ("pastebinit isn't properly installed, this extension won't work")

    def show_error_message(self, error, description):
        if Notify == None:
            md = Gtk.MessageDialog(None, 0,
                                   Gtk.MessageType.ERROR, Gtk.ButtonsType.CLOSE,
                                   error)
            md.format_secondary_text(description)
            md.run()
            md.destroy()
            return
        n = Notify.Notification.new(error, description, None)
        helper = Gtk.Button()
        icon = helper.render_icon(Gtk.STOCK_DIALOG_ERROR, Gtk.IconSize.DIALOG)
        n.set_image_from_pixbuf(icon)
        n.show()
    
    def get_file_items(self, window, files):
        if not self.presets:
            return
        if len(files)!=1:
            return
        filename = files[0]

        if filename.get_uri_scheme() != 'file' or filename.is_directory():
            return

        items = []
        pastebin = self.settings.get_string("pastebin")
        #Called when the user selects a file in Nautilus.
        item = Nautilus.MenuItem(name="NautilusPython::pastebin_item",
                                 label=_("Pastebin to %s" % pastebin),
                                 tip=_("Send this file to %s" % pastebin))
        item.set_property('icon', "nautilus-pastebin")
        item.connect("activate", self.menu_activate_cb, files)
        items.append(item)
        return items

    def menu_activate_cb(self, menu, files):
        if len(files) != 1:
            return
        filename = files[0]

        # Make sure "pastebin" value is supported by pastebinit
        pastebin = self.settings.get_string("pastebin")
        if not pastebin in self.presets:
            log (_("ERROR: The selected pastebin is not valid."))
            return

        # TODO: MIME check

        filename = urllib.unquote(filename.get_uri()[7:])

        # Don't paste empty files
        size = os.stat(filename).st_size
        if not size > 0:
            self.show_error_message(_("Error"), _("Unable to send an empty file."))
            return

        # Check whether the file is plain text
        output = check_output(['file', '-i', filename])
        if "charset=binary" in output:
            self.show_error_message(_("Error"), _("Unable to send binary files."))
            return

        if self.settings.get_boolean("askconfirmation"):
            md = Gtk.MessageDialog(None, Gtk.DialogFlags.MODAL,
                                   Gtk.MessageType.QUESTION, Gtk.ButtonsType.YES_NO,
                                   _("The file will be sent to the selected pastebin: are you sure?"))
            response = md.run()
            if response == Gtk.ResponseType.NO:
                md.destroy()
                return

            md.destroy()

        thread = PastebinThread(self.settings, filename)
        thread.start()
        while (thread.isAlive()):
            time.sleep(0.09)
            while Gtk.events_pending():
                Gtk.main_iteration()
